%global python3_pkgversion 3.11
%global newname %{name}py%{python3_pkgversion}
Summary:        A high-performance implementation of MPI
Name:           mpich
Version:        4.2.1
Release:        3.ac14.py3.11%{?dist}
License:        mpich2
URL:            https://www.mpich.org/

Source0:        https://www.mpich.org/static/downloads/%{version}/%{name}-%{version}.tar.gz
Source1:        mpich.macros
Source3:        mpich.pth.py3

Patch:          0001-pkgconf-remove-optimization-and-link-flags-from-pkgc.patch
Patch:          0002-pkgconf-also-drop-rpath-flags-from-pkgconf-file.patch
Patch:          0003-Drop-build-flags-e.g.-specs.-and-lto-from-mpi-wrappe.patch
Patch:          0004-Make-mpich.module-useful.patch
# TODO: submit ^ upstream

Patch:          mpich-configure-max_align_t.patch
Patch:          mpich-aclocal_cc-implicit-int.patch
Patch:          mpich-json-configure-__thread.patch

BuildRequires:  make
BuildRequires:  autoconf
BuildRequires:  automake
BuildRequires:  libtool
BuildRequires:  gcc
BuildRequires:  gcc-c++
BuildRequires:  gcc-gfortran
BuildRequires:  hwloc-devel >= 2.0
%if ! (0%{?rhel} >= 10)
%ifarch x86_64
# BuildRequires:  json-c-devel
BuildRequires:  libpsm2-devel
%endif
%endif
BuildRequires:  libfabric-devel
BuildRequires:  libnl3-devel
BuildRequires:  libuuid-devel
BuildRequires:  numactl-devel
%ifarch aarch64 ppc64le x86_64
BuildRequires:  ucx-devel
%endif
%if ! 0%{?rhel}
BuildRequires:  yaksa-devel
%else
Provides:       bundled(yaksa) = 0.2
%endif
# For ./maint/extractcvars
BuildRequires:  perl(lib)
%ifarch %{valgrind_arches}
BuildRequires:  valgrind-devel
%endif
# For %%{python3_sitearch}
BuildRequires:  python%{python3_pkgversion}-devel
BuildRequires:  rpm-mpi-hooks
Provides:       mpi
Provides:       mpich2 = %{version}
Obsoletes:      mpich2 < 3.0
Requires:       environment(modules)

# Make sure this package is rebuilt with correct Python version when updating
# Otherwise mpi.req from rpm-mpi-hooks doesn't work
# https://bugzilla.redhat.com/show_bug.cgi?id=1705296
Requires:       (python(abi) = %{python3_version} if python3)

%description
MPICH is a high-performance and widely portable implementation of the Message
Passing Interface (MPI) standard (MPI-1, MPI-2 and MPI-3). The goals of MPICH
are: (1) to provide an MPI implementation that efficiently supports different
computation and communication platforms including commodity clusters (desktop
systems, shared-memory systems, multicore architectures), high-speed networks
(10 Gigabit Ethernet, InfiniBand, Myrinet, Quadrics) and proprietary high-end
computing systems (Blue Gene, Cray) and (2) to enable cutting-edge research in
MPI through an easy-to-extend modular framework for other derived
implementations.

The mpich binaries in this RPM packages were configured to use the default
process manager (Hydra) using the default device (ch3). The ch3 device
was configured with support for the nemesis channel that allows for
shared-memory and TCP/IP sockets based communication.

This build also include support for using the 'module environment' to select
which MPI implementation to use when multiple implementations are installed.
If you want MPICH support to be automatically loaded, you need to install the
mpich-autoload package.

%package  -n %{newname}-autoload
Summary:        Load mpich automatically into profile
Requires:       mpich = %{version}-%{release}
Provides:       mpich2-autoload = 3.0.1
Obsoletes:      mpich2-autoload < 3.0

%description  -n %{newname}-autoload
This package contains profile files that make mpich automatically loaded.

%package  -n %{newname}-devel
Summary:        Development files for mpich
Provides:       %{name}-devel-static = %{version}-%{release}
Requires:       %{newname} = %{version}-%{release}
Requires:       pkgconfig
Requires:       gcc-gfortran
Requires:       rpm-mpi-hooks
Requires:       redhat-rpm-config
Provides:       mpich2-devel = 3.0.1
Obsoletes:      mpich2-devel < 3.0

%description  -n %{newname}-devel
Contains development headers and libraries for mpich

%package  -n %{newname}-doc
Summary:        Documentations and examples for mpich
BuildArch:      noarch
Requires:       %{newname}-devel = %{version}-%{release}
Provides:       mpich2-doc = 3.0.1
Obsoletes:      mpich2-doc < 3.0

%description  -n %{newname}-doc
Contains documentations, examples and man-pages for mpich

%package -n python%{python3_pkgversion}-mpich
Summary:        mpich support for Python 3
Requires:       %{newname} = %{version}-%{release}
Requires:       python(abi) = %{python3_version}

%description -n python%{python3_pkgversion}-mpich
mpich support for Python 3.

%package -n %{newname}
Summary:        %{summary}

%description -n %{newname}

Description

%prep
%autosetup -p1

%build
./autogen.sh

CONFIGURE_OPTS=(
        --with-custom-version-string=%{version}-%{release}
        --enable-sharedlibs=gcc
        --enable-shared
        --enable-static=no
        --enable-lib-depend
        --disable-rpath
        --disable-silent-rules
        --disable-dependency-tracking
        --with-gnu-ld
        --with-pm=hydra:gforker
        --includedir=%{_includedir}/%{newname}-%{_arch}
        --bindir=%{_libdir}/%{newname}/bin
        --libdir=%{_libdir}/%{newname}/lib
        --datadir=%{_datadir}/%{newname}
        --mandir=%{_mandir}/%{newname}-%{_arch}
        --docdir=%{_datadir}/%{newname}/doc
        --htmldir=%{_datadir}/%{newname}/doc
        --with-hwloc
        --with-libfabric
%ifarch aarch64 ppc64le x86_64
        --with-ucx
%endif
%if ! 0%{?rhel}
        --with-yaksa
%endif
)
#        --with-device=ch3:nemesis

# Set -fallow-argument-mismatch for #1795817
%configure "${CONFIGURE_OPTS[@]}"               \
  FFLAGS="$FFLAGS -fallow-argument-mismatch"    \
  FCFLAGS="$FCFLAGS -fallow-argument-mismatch"

# Remove rpath
sed -r -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' libtool
sed -r -i 's|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' libtool

#Try and work around 'unused-direct-shlib-dependency' rpmlint warnning
sed -i -e 's| -shared | -Wl,--as-needed\0|g' libtool

%make_build VERBOSE=1

%install
%make_install

mkdir -p %{buildroot}%{_fmoddir}/%{newname}
mv  %{buildroot}%{_includedir}/%{newname}-*/*.mod %{buildroot}%{_fmoddir}/%{newname}/
sed -r -i 's|^modincdir=.*|modincdir=%{_fmoddir}/%{newname}|' %{buildroot}%{_libdir}/%{newname}/bin/mpifort

# Install the module file
mkdir -p %{buildroot}%{_datadir}/modulefiles/mpi
sed -r 's|%{_bindir}|%{_libdir}/%{newname}/bin|;
        s|@LIBDIR@|%{_libdir}/%{newname}|;
        s|@MPINAME@|%{name}|;
        s|@py2sitearch@|%{python2_sitearch}|;
        s|@py3sitearch@|%{python3_sitearch}|;
        s|@ARCH@|%{_arch}|;
        s|@fortranmoddir@|%{_fmoddir}|;
     ' \
     <src/packaging/envmods/mpich.module \
     >%{buildroot}%{_datadir}/modulefiles/mpi/%{newname}-%{_arch}

mkdir -p %{buildroot}%{_sysconfdir}/profile.d
cat >%{buildroot}%{_sysconfdir}/profile.d/mpich-%{_arch}.sh <<EOF
# Load mpich environment module
module load mpi/%{name}-%{_arch}
EOF
cp -p %{buildroot}%{_sysconfdir}/profile.d/mpich-%{_arch}.{sh,csh}

# Install the RPM macros
install -pDm0644 %{SOURCE1} %{buildroot}%{_rpmconfigdir}/macros.d/macros.%{newname}

# Install the .pth files
mkdir -p %{buildroot}%{python3_sitearch}/%{name}
install -pDm0644 %{SOURCE3} %{buildroot}%{python3_sitearch}/%{name}.pth

# Create cmake directory
mkdir -p %{buildroot}%{_libdir}/%{newname}/lib/cmake/

# Create directories for MPICH application development files
mkdir -p %{buildroot}%{_libdir}/%{newname}/lib/cmake
mkdir -p %{buildroot}%{_libdir}/%{newname}/include

find %{buildroot} -type f -name "*.la" -delete

%check
make check VERBOSE=1 \
%ifarch ppc64le
|| :
%endif
# The test results are ignored on ppc64le. The tests started failing
# in the bundled openpa checksuite. Upstream has already removed it,
# so the issue should resolve itself for the next release and I don't
# think it's worth the time to solve it here.

%ldconfig_scriptlets

%files -n %{newname}
%license COPYRIGHT
%doc CHANGES README README.envvar RELEASE_NOTES
%dir %{_libdir}/%{newname}
%dir %{_libdir}/%{newname}/lib
%dir %{_libdir}/%{newname}/bin
%dir %{_libdir}/%{newname}/lib/cmake
%dir %{_libdir}/%{newname}/include
%dir %{_fmoddir}/%{newname}
%{_libdir}/%{newname}/lib/*.so.*
%{_libdir}/%{newname}/bin/hydra*
%{_libdir}/%{newname}/bin/mpichversion
%{_libdir}/%{newname}/bin/mpiexec*
%{_libdir}/%{newname}/bin/mpirun
%{_libdir}/%{newname}/bin/mpivars
%{_libdir}/%{newname}/bin/parkill
%dir %{_mandir}/%{newname}-%{_arch}
%doc %{_mandir}/%{newname}-%{_arch}/man1/
%{_datadir}/modulefiles/mpi/

%files  -n %{newname}-autoload
%{_sysconfdir}/profile.d/mpich-%{_arch}.*

%files  -n %{newname}-devel
%{_includedir}/%{newname}-%{_arch}/
%{_libdir}/%{newname}/lib/pkgconfig/
%{_libdir}/%{newname}/lib/cmake/
%{_libdir}/%{newname}/lib/*.so
%{_libdir}/%{newname}/bin/mpicc
%{_libdir}/%{newname}/bin/mpic++
%{_libdir}/%{newname}/bin/mpicxx
%{_libdir}/%{newname}/bin/mpif77
%{_libdir}/%{newname}/bin/mpif90
%{_libdir}/%{newname}/bin/mpifort
%{_fmoddir}/%{newname}/
%{_rpmconfigdir}/macros.d/macros.%{newname}
%{_mandir}/%{newname}-%{_arch}/man3/

%files  -n %{newname}-doc
%dir %{_datadir}/%{newname}
%{_datadir}/%{newname}/doc/

%files -n python%{python3_pkgversion}-mpich
%dir %{python3_sitearch}/%{name}
%{python3_sitearch}/%{name}.pth

%changelog
* Tue Apr 23 2024 Ding-Yi Chen <dingyichen@gmail.com> -  4.2.1-3.ac14.py3.11
 - Cut corner packaging
